<?php

namespace Drupal\inactive_user_management\Notification;

use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\inactive_user_management\Settings\GetNotificationSettings;
use Drupal\symfony_mailer\EmailFactoryInterface;
use Drupal\user\UserInterface;

/**
 * The InactiveUserNotification service class.
 */
class InactiveUserNotification {

  /**
   * The EmailFactory service object.
   *
   * @var \Drupal\symfony_mailer\EmailFactoryInterface
   */
  private EmailFactoryInterface $emailFactory;

  /**
   * The EntityTypeManager service object.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The LoggerChannelFactory service object.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  private LoggerChannelFactoryInterface $loggerFactory;

  /**
   * The Time service object.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  private TimeInterface $time;

  /**
   * The GetNotificationSettings service object.
   *
   * @var \Drupal\inactive_user_management\Settings\GetNotificationSettings
   */
  private GetNotificationSettings $notificationSettings;

  /**
   * The QueueFactory object.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  private QueueInterface $queue;

  public function __construct(
    EmailFactoryInterface $email_factory,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory,
    TimeInterface $time,
    GetNotificationSettings $get_notification_settings,
    QueueFactory $queue,
  ) {
    $this->emailFactory = $email_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->loggerFactory = $logger_factory;
    $this->time = $time;
    $this->notificationSettings = $get_notification_settings;
    $this->queue = $queue->get('inactive_user_notification');
  }

  /**
   * Sends an email notification to users who weren't active for a given time.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function sendNotificationToInactiveUsers(): void {
    if ($this->notificationSettings->getState()) {
      $users = $this->getAllUsersToNotify();

      $this->queue->createQueue();
      if (!empty($users)) {
        /** @var \Drupal\user\UserInterface $user */
        foreach ($users as $user) {
          $this->queue->createItem([
            'id' => $user->id(),
            'user' => $user,
          ]);
        }
      }
    }
  }

  /**
   * Sends the email notification to the passed user.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user which receives a notification.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function sendNotification(UserInterface $user): void {
    $email = $this->emailFactory->sendTypedEmail('inactive_user_mailer',
      'notification',
      $user->getEmail(),
      $this->notificationSettings->getSubject(),
      $this->notificationSettings->getContent());
    if (!$email->getError()) {
      $user->set('field_last_notification', $this->time->getCurrentTime())
        ->save();
      $this->loggerFactory->get('Inactive User Management')
        ->info('There was a notification sent to the inactive user: {email}', ['email' => $user->getEmail()]);

    }
    else {
      $this->loggerFactory->get('Inactive User Management')
        ->error("An error occurred while trying to send a notification to the inactive user: " . $user->getEmail());
    }
  }

  /**
   * Gets a list of all users who need to be notified, based on the settings.
   *
   * @return array|null
   *   Returns all users as array, or null if no user need to be notified.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getAllUsersToNotify(): array|NULL {
    $userQuery = $this->entityTypeManager->getStorage('user')->getQuery();
    $group = $userQuery
      ->orConditionGroup()
      ->condition('field_last_notification', $this->getCompareTimeForLastNotification(), '<')
      ->notExists('field_last_notification');

    $userQuery
      ->accessCheck(FALSE)
      ->condition('status', 1)
      ->condition('roles', array_values($this->notificationSettings->getRoles()), 'IN')
      ->condition($this->notificationSettings->getUserField(), $this->getCompareTimeForActiveness(), '<')
      ->condition('created', $this->getCompareTimeForActiveness(), '<')
      ->condition($group);

    if (!$this->notificationSettings->getIncludeNeverAccessed()) {
      $userIds = $userQuery
        ->accessCheck(FALSE)
        ->condition('access', 0, '>')
        ->execute();
    }
    else {
      $userIds = $userQuery->accessCheck(FALSE)->execute();
    }

    $users = $this->entityTypeManager->getStorage('user')->loadMultiple($userIds);

    if (!empty($users)) {
      return $users;
    }
    return NULL;
  }

  /**
   * Gets the time comparative of now and the timespan between notifications.
   *
   * @return int
   *   Returns the timestamp as int.
   */
  public function getCompareTimeForLastNotification(): int {
    return $this->time->getCurrentTime() - $this->notificationSettings->getTimespanBetweenNotifications();
  }

  /**
   * Gets the time comparative of now and the timespan of inactivity.
   *
   * @return int
   *   Returns the timestamp as int.
   */
  public function getCompareTimeForActiveness(): int {
    return $this->time->getCurrentTime() - $this->notificationSettings->getTimespan();
  }

}
