<?php

namespace Drupal\inactive_user_management\Settings;

use Drupal\Core\State\StateInterface;

/**
 * The GetNotificationSettings service class.
 */
class GetNotificationSettings {

  /**
   * The state object.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  private StateInterface $state;

  public function __construct(StateInterface $state) {
    $this->state = $state;
  }

  /**
   * Gets all settings of the notification service.
   *
   * @return array
   *   Returns and array with all the notification settings.
   */
  public function getAllNotificationSettings(): array {
    return [
      'enabled' => $this->getState(),
      'roles' => $this->getRoles(),
      'never_accessed' => $this->getIncludeNeverAccessed(),
      'user_field' => $this->getUserField(),
      'timespan' => $this->getTimespan(),
      'timespan_notification' => $this->getTimespanBetweenNotifications(),
      'subject' => $this->getSubject(),
      'content' => $this->getContent(),
    ];
  }

  /**
   * Checks if the notification service is enabled.
   *
   * @return int
   *   Returns 1 if the notification service is enabled and 0 if not.
   */
  public function getState(): int {
    return $this->state->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_enabled');
  }

  /**
   * Gets all the roles which should receive a notification.
   *
   * @return array
   *   Returns all the configured roles in an array.
   */
  public function getRoles(): array {
    $roles = $this->state->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_roles');
    $roleArray = [];
    foreach ($roles as $role) {
      if ($role != 0) {
        $roleArray[] = $role;
      }
    }
    return $roleArray;
  }

  /**
   * Checks if users who never accessed should be included.
   *
   * @return int
   *   Returns 1 if users who never accessed should be included or 0 if not.
   */
  public function getIncludeNeverAccessed(): int {
    return $this->state->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_never_accessed');
  }

  /**
   * Checks if the last access or last login time should be used as reference.
   *
   * @return string
   *   Returns the name of the user field as string.
   */
  public function getUserField(): string {
    return $this->state->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_user_field');
  }

  /**
   * Gets the timespan of inactivity after which the user will be notified.
   *
   * @return string
   *   Returns the configured timespan in seconds as string.
   */
  public function getTimespan(): string {
    return $this->state->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan');
  }

  /**
   * Gets the configured timespan between two notifications.
   *
   * @return string
   *   Returns the timespan in seconds as string.
   */
  public function getTimespanBetweenNotifications(): string {
    return $this->state->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan_last_notification');
  }

  /**
   * Gets the email subject.
   *
   * @return string
   *   Returns the configured email subject as string.
   */
  public function getSubject(): string {
    return $this->state->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_subject');
  }

  /**
   * Gets the email content.
   *
   * @return string
   *   Returns the configured email content as string.
   */
  public function getContent(): string {
    return $this->state->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_content');
  }

}
